Package: modeltime
Title: The Tidymodels Extension for Time Series Modeling
Version: 1.3.1
Authors@R: c(
    person("Matt", "Dancho", email = "mdancho@business-science.io", role = c("aut", "cre")),
    person("Business Science", role = "cph"))
Description: 
    The time series forecasting framework for use with the 'tidymodels' ecosystem. 
    Models include ARIMA, Exponential Smoothing, and additional time series models
    from the 'forecast' and 'prophet' packages. Refer to "Forecasting Principles & Practice, Second edition" 
    (<https://otexts.com/fpp2/>).
    Refer to "Prophet: forecasting at scale" 
    (<https://research.facebook.com/blog/2017/02/prophet-forecasting-at-scale/>.).
URL: https://github.com/business-science/modeltime,
        https://business-science.github.io/modeltime/
BugReports: https://github.com/business-science/modeltime/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Imports: StanHeaders, timetk (>= 2.8.1), parsnip (>= 0.2.1), dials,
        yardstick (>= 0.0.8), workflows (>= 1.0.0), hardhat (>= 1.0.0),
        rlang (>= 0.1.2), glue, plotly, reactable, gt, ggplot2, tibble,
        tidyr, dplyr (>= 1.1.0), purrr, stringr, forcats, scales,
        janitor, parallel, parallelly, doParallel, foreach, magrittr,
        forecast, xgboost (>= 1.2.0.1), prophet, methods, cli,
        tidymodels
Suggests: rstan, slider, sparklyr, workflowsets, recipes, rsample, tune
        (>= 0.2.0), lubridate, testthat, kernlab, glmnet, thief,
        smooth, greybox, earth, randomForest, trelliscopejs, knitr,
        rmarkdown (>= 2.9), webshot, qpdf, TSrepr
VignetteBuilder: knitr
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2024-10-22 19:42:04 UTC; mdancho
Author: Matt Dancho [aut, cre],
  Business Science [cph]
Maintainer: Matt Dancho <mdancho@business-science.io>
Repository: CRAN
Date/Publication: 2024-10-22 20:10:02 UTC
Built: R 4.4.1; ; 2024-11-04 03:36:12 UTC; unix
